'\"
'\" Copyright (c) 1990-1994 The Regents of the University of California.
'\" Copyright (c) 1994-1995 Sun Microsystems, Inc.
'\"
'\" See the file "license.terms" for information on usage and redistribution
'\" of this file, and for a DISCLAIMER OF ALL WARRANTIES.
'\" 
'\" @(#) button.n 1.33 95/08/12 17:35:02
'\" 
.so STk-man.macros
.TH button n 4.0 STk "Tk Built-In Commands"
.BS
'\" Note:  do not modify the .SH NAME line immediately below!
.SH NAME
button \- Create and manipulate button widgets
.SH SYNOPSIS
(\fBbutton\fI \fIwidget\-name \fR?\fIoptions\fR?)
.SO
:activebackground	:cursor	:highlightthickness	:takefocus
:activeforeground	:disabledforeground	:image	:text
:anchor	:font	:justify	:textvariable
:background	:foreground	:padx	:underline
:bitmap	:highlightbackground	:pady	:wraplength
:borderwidth	:highlightcolor	:relief
.SE
.SH "WIDGET-SPECIFIC OPTIONS"
.OP :command command Command command
Specifies a STk procedure to associate with the button.  This procedure
is typically invoked when mouse button 1 is released over the button
window.
OP :default default Default default
.VS
Specifies one of three states for the default ring: \fB"normal"\fR,
\fB"active"\fR, or \fB"disabled"\fR.  In active state, the button is drawn
with the platform specific appearance for a default button.  In normal
state, the button is drawn with the platform specific appearance for a
non-default button, leaving enough space to draw the default button
appearance.  The normal and active states will result in buttons of
the same size.  In disabled state, the button is drawn with the
non-default button appearance without leaving space for the default
appearance.  The disabled state may result in a smaller button than
the active state.
ring.
.OP :environment environment Environment environment
Specifies the environment in which the \fB:textvariable\fR must be taken. 
By default, the value of this option is the STk global environment.
.VE
.OP :height height Height height
Specifies a desired height for the button.
If an image or bitmap is being displayed in the button then the value is in
screen units (i.e. any of the forms acceptable to \fBTk_GetPixels\fR);
for text it is in lines of text.
If this option isn't specified, the button's desired height is computed
from the size of the image or bitmap or text being displayed in it.
.OP :state state State state
Specifies one of three states for the button:  \fBnormal\fR, \fBactive\fR,
or \fBdisabled\fR.  In normal state the button is displayed using the
\fBforeground\fR and \fBbackground\fR options.  The active state is
typically used when the pointer is over the button.  In active state
the button is displayed using the \fBactiveForeground\fR and
\fBactiveBackground\fR options.  Disabled state means that the button
should be insensitive:  the default bindings will refuse to activate
the widget and will ignore mouse button presses.
In this state the \fBdisabledForeground\fR and
\fBbackground\fR options determine how the button is displayed.
.OP :width width Width width
Specifies a desired width for the button.
If an image or bitmap is being displayed in the button then the value is in
screen units (i.e. any of the forms acceptable to \fBTk_GetPixels\fR);
for text it is in characters.
If this option isn't specified, the button's desired width is computed
from the size of the image or bitmap or text being displayed in it.
.BE

.SH DESCRIPTION
.PP
The \fBbutton\fR procedure creates a new window (given by the
\fIwidget\-name\fR argument) and makes it into a button widget.
Additional
options, described above, may be specified on the procedure line
or in the option database
to configure aspects of the button such as its colors, font,
text, and initial relief.  The \fBbutton\fR procedure returns its
\fIwidget\-name\fR argument.  At the time this procedure is invoked,
there must not exist a window named \fIwidget\-name\fR, but
\fIwidget\-name\fR's parent must exist.
.PP
A button is a widget that displays a textual string, bitmap or image.
If text is displayed, it must all be in a single font, but it
can occupy multiple lines on the screen (if it contains newlines
or if wrapping occurs because of the \fBwrapLength\fR option) and
one of the characters may optionally be underlined using the
\fBunderline\fR option.
It can display itself in either of three different ways, according
to
the \fBstate\fR option; 
it can be made to appear raised, sunken, or flat;
and it can be made to flash.  When a user invokes the
button (by pressing mouse button 1 with the cursor over the
button), then the STk procedure specified in the \fB:command\fR
option is invoked.

.SH "WIDGET PROCEDURE"
.PP
The \fBbutton\fR procedure creates a new STk procedure whose
name is \fIwidget\-name\fR.  This
procedure may be used to invoke various
operations on the widget.  It has the following general form:
.CS
(\fIwidget\-name option \fR?\fIarg arg ...\fR?)
.CE
\fIOption\fR and the \fIarg\fRs
determine the exact behavior of the procedure.  The following
procedures are possible for button widgets:
.TP
(\fIwidget\-name \fB'cget\fR \fIoption\fR)
Returns the current value of the configuration option given
by \fIoption\fR.
\fIOption\fR may have any of the values accepted by the \fBbutton\fR
procedure.
.TP
(\fIwidget\-name \fB'configure\fR ?\fIoption\fR? ?\fIvalue option value ...\fR?)
Query or modify the configuration options of the widget.
If no \fIoption\fR is specified, returns a list describing all of
the available options for \fIwidget\-name\fR (see \fBTk_ConfigureInfo\fR for
information on the format of this list).  If \fIoption\fR is specified
with no \fIvalue\fR, then the procedure returns a list describing the
one named option (this list will be identical to the corresponding
sublist of the value returned if no \fIoption\fR is specified).  If
one or more \fIoption\-value\fR pairs are specified, then the procedure
modifies the given widget option(s) to have the given value(s);  in
this case the procedure returns an empty string.
\fIOption\fR may have any of the values accepted by the \fBbutton\fR
procedure.
.TP
(\fIwidget\-name \fB'flash\fR)
Flash the button.  This is accomplished by redisplaying the button
several times, alternating between active and normal colors.  At
the end of the flash the button is left in the same normal/active
state as when the procedure was invoked.
This procedure is ignored if the button's state is \fBdisabled\fR.
.TP
(\fIwidget\-name \fB'invoke\fR)
Invoke the STk procedure associated with the button, if there is one
(return value is undefined).
This procedure is ignored if the button's state is \fBdisabled\fR.

.SH "DEFAULT BINDINGS"
.PP
Tk automatically creates class bindings for buttons that give them
default behavior:
.IP [1]
A button activates whenever the mouse passes over it and deactivates
whenever the mouse leaves the button.
.VS
Under Windows, this binding is only active when mouse button 1 has
been pressed over the button.
.VE
.IP [2]
A button's relief is changed to sunken whenever mouse button 1 is
pressed over the button, and the relief is restored to its original
value when button 1 is later released.
.IP [3]
If mouse button 1 is pressed over a button and later released over
the button, the button is invoked.  However, if the mouse is not
over the button when button 1 is released, then no invocation occurs.
.IP [4]
When a button has the input focus, the space key causes the button
to be invoked.
.PP
If the button's state is \fBdisabled\fR then none of the above
actions occur:  the button is completely non-responsive.
.PP
The behavior of buttons can be changed by defining new bindings for
individual widgets or by redefining the class bindings.

.SH SEE ALSO
checkbutton, radiobutton
