'\"
'\" Copyright (c) 1990-1994 The Regents of the University of California.
'\" Copyright (c) 1994-1995 Sun Microsystems, Inc.
'\"
'\" See the file "license.terms" for information on usage and redistribution
'\" of this file, and for a DISCLAIMER OF ALL WARRANTIES.
'\" 
'\" @(#) radiobutton.n 1.36 95/08/12 17:35:17
'\" 
.so STk-man.macros
.TH radiobutton n 3.1 STk "Tk Built-In Commands"
.BS
'\" Note:  do not modify the .SH NAME line immediately below!
.SH NAME
radiobutton \- Create and manipulate radiobutton widgets
.SH SYNOPSIS
(\fBradiobutton\fI \fIwidget\-name \fR?\fIoptions\fR?-
.SO
:activebackground	:cursor	:highlightthickness	:takefocus
:activeforeground	:disabledforeground	:image	:text
:anchor	:font	:justify	:textvariable
:background	:foreground	:padx	:underline
:bitmap	:highlightbackground	:pady	:wraplength
:borderwidth	:highlightcolor	:relief
.SE
.SH "WIDGET-SPECIFIC OPTIONS"
.OP :command command Command command
Specifies a STk procedure to associate with the button.  This procedure
is typically invoked when mouse button 1 is released over the button
window.  The button's global variable (\fB:variable\fR option) will
be updated before the procedure is invoked.
.OP :height height Height height
Specifies a desired height for the button.
If an image or bitmap is being displayed in the button then the value is in
screen units (i.e. any of the forms acceptable to \fBTk_GetPixels\fR);
for text it is in lines of text.
If this option isn't specified, the button's desired height is computed
from the size of the image or bitmap or text being displayed in it.
.OP :indicatoron indicatorOn IndicatorOn indicator-on
Specifies whether or not the indicator should be drawn.  Must be a
proper boolean value.  If false, the \fBrelief\fR option is
ignored and the widget's relief is always sunken if the widget is
selected and raised otherwise.
.OP :selectcolor selectColor Background select-color
Specifies a background color to use when the button is selected.
If \fBindicatorOn\fR is true, the color applicies to the indicator.
If \fBindicatorOn\fR is false, this color is used as the background
for the entire widget, in place of \fBbackground\fR or \fBactiveBackground\fR,
whenever the widget is selected.
If specified as an empty string, no special color is used for
displaying when the widget is selected.
.OP :selectimage selectImage SelectImage select-image
Specifies an image to display (in place of the \fBimage\fR option)
when the radiobutton is selected.
This option is ignored unless the \fBimage\fR option has been
specified.
.OP :state state State state
Specifies one of three states for the radiobutton:  \fBnormal\fR, \fBactive\fR,
or \fBdisabled\fR.  In normal state the radiobutton is displayed using the
\fBforeground\fR and \fBbackground\fR options.  The active state is
typically used when the pointer is over the radiobutton.  In active state
the radiobutton is displayed using the \fBactiveForeground\fR and
\fBactiveBackground\fR options.  Disabled state means that the radiobutton
should be insensitive:  the default bindings will refuse to activate
the widget and will ignore mouse button presses.
In this state the \fBdisabledForeground\fR and
\fBbackground\fR options determine how the radiobutton is displayed.
.OP :stringvalue stringvalue StringValue string-value
Specifies if the value set to \fBvariable\fR by \fB:value\fR must be 
\fIstringified\fR. Default value for this option is \fB#t\fR. 
See \fBcheckbutton\fR for a similar example.
.OP :value value Value  value
Specifies value to store in the button's associated variable whenever
this button is selected.
.OP :variable variable Variable variable
Specifies name of global variable to set whenever this button is
selected.  Changes in this variable also cause the button to select
or deselect itself.
Defaults to the value \fB*selected-button*\fR.
.OP :width width Width width
Specifies a desired width for the button.
If an image or bitmap is being displayed in the button, the value is in
screen units (i.e. any of the forms acceptable to \fBTk_GetPixels\fR);
for text it is in characters.
If this option isn't specified, the button's desired width is computed
from the size of the image or bitmap or text being displayed in it.
.BE

.SH DESCRIPTION
.PP
The \fBradiobutton\fR procedure creates a new window (given by the
\fIwidget\-name\fR argument) and makes it into a radiobutton widget.
Additional
options, described above, may be specified on the procedure line
or in the option database
to configure aspects of the radiobutton such as its colors, font,
text, and initial relief.  The \fBradiobutton\fR procedure returns its
\fIwidget\-name\fR argument.  At the time this procedure is invoked,
there must not exist a window named \fIwidget\-name\fR, but
\fIwidget\-name\fR's parent must exist.
.PP
A radiobutton is a widget
that displays a textual string, bitmap or image
and a diamond called an \fIindicator\fR.
If text is displayed, it must all be in a single font, but it
can occupy multiple lines on the screen (if it contains newlines
or if wrapping occurs because of the \fBwrapLength\fR option) and
one of the characters may optionally be underlined using the
\fBunderline\fR option.
A radiobutton has
all of the behavior of a simple button: it can display itself in either
of three different ways, according to the \fBstate\fR option;
it can be made to appear
raised, sunken, or flat; it can be made to flash; and it invokes
a STk procedure whenever mouse button 1 is clicked over the
check button.
.PP
In addition, radiobuttons can be \fIselected\fR.
If a radiobutton is selected, the indicator is normally
drawn with a sunken relief and a special color, and
a STk variable associated with the radiobutton is set to a particular
value.
If the radiobutton is not selected, the indicator is drawn with a
raised relief and no special color.
Typically, several radiobuttons share a single variable and the
value of the variable indicates which radiobutton is to be selected.
When a radiobutton is selected it sets the value of the variable to
indicate that fact;  each radiobutton also monitors the value of
the variable and automatically selects and deselects itself when the
variable's value changes.
By default the variable \fB*selected-button*\fR
is used;  its contents give the name of the button that is
selected, or the empty string if no button associated with that
variable is selected.
The name of the variable for a radiobutton,
plus the variable to be stored into it, may be modified with options
on the procedure line or in the option database.
Configuration options may also be used to modify the way the
indicator is displayed (or whether it is displayed at all).
By default a radio
button is configured to select itself on button clicks.

.SH "WIDGET PROCEDURE"
.PP
The \fBradiobutton\fR procedure creates a new STk procedure whose
name is \fIwidget\-name\fR.  This
procedure may be used to invoke various
operations on the widget.  It has the following general form:
.CS
(\fIwidget\-name option \fR?\fIarg arg ...\fR?)
.CE
\fIOption\fR and the \fIarg\fRs
determine the exact behavior of the procedure.  The following
procedures are possible for radiobutton widgets:
.TP
(\fIwidget\-name '\fBcget\fR \fIoption\fR)
Returns the current value of the configuration option given
by \fIoption\fR.
\fIOption\fR may have any of the values accepted by the \fBradiobutton\fR
procedure.
.TP
(\fIwidget\-name '\fBconfigure\fR ?\fIoption\fR? ?\fIvalue option value ...\fR?)
Query or modify the configuration options of the widget.
If no \fIoption\fR is specified, returns a list describing all of
the available options for \fIwidget\-name\fR (see \fBTk_ConfigureInfo\fR for
information on the format of this list).  If \fIoption\fR is specified
with no \fIvalue\fR, the procedure returns a list describing the
one named option (this list will be identical to the corresponding
sublist of the value returned if no \fIoption\fR is specified).  If
one or more \fIoption\-value\fR pairs are specified, the procedure
modifies the given widget option(s) to have the given value(s);  in
this case the procedure returns an empty string.
\fIOption\fR may have any of the values accepted by the \fBradiobutton\fR
procedure.
.TP
(\fIwidget\-name '\fBdeselect\fR)
Deselects the radiobutton and sets the associated variable to an
empty string.
If this radiobutton was not currently selected, the procedure has
no effect.
.TP
(\fIwidget\-name '\fBflash\fR)
Flashes the radiobutton.  This is accomplished by redisplaying the radiobutton
several times, alternating between active and normal colors.  At
the end of the flash the radiobutton is left in the same normal/active
state as when the procedure was invoked.
This procedure is ignored if the radiobutton's state is \fBdisabled\fR.
.TP
(\fIwidget\-name '\fBinvoke\fR)
Does just what would have happened if the user invoked the radiobutton
with the mouse: selects the button and invokes
its associated STk procedure, if there is one (return value is undifined)..
This procedure is ignored if the radiobutton's state is \fBdisabled\fR.
.TP
(\fIwidget\-name '\fBselect\fR)
Selects the radiobutton and sets the associated variable to the
value corresponding to this widget.

.SH BINDINGS
.PP
Tk automatically creates class bindings for radiobuttons that give them
the following default behavior:
.IP [1]
The radiobutton activates whenever the mouse passes over it and deactivates
whenever the mouse leaves the radiobutton.
.IP [2]
When mouse button 1 is pressed over a radiobutton it is invoked (it
becomes selected and the procedure associated with the button is
invoked, if there is one).
.IP [3]
When a radiobutton has the input focus, the space key causes the radiobutton
to be invoked.
.PP
If the radiobutton's state is \fBdisabled\fR then none of the above
actions occur:  the radiobutton is completely non-responsive.
.PP
The behavior of radiobuttons can be changed by defining new bindings for
individual widgets or by redefining the class bindings.

.SH SEE ALSO
button, checkbutton
